# -*- coding: utf-8 -*-
"""
Created on Tue Dec 10 12:48:31 2019

@author: Vikram Khanna
"""

import numpy as np
import random
from scipy.optimize import fsolve
import matplotlib.pyplot as plt
###############################################################################
def file2array(FileName):
    """
    function:
        file2array
    arguments:
        FileName, the name of the file to be read (including the path: in case it is not in the cwd)
    returns:
        fileData, the numpy array (N_lines-1,N_words) here (24,3), columns: N, U, Iters
        UMin, the minimum energy value
    """
    print("READING FILE...")  
    f=open(FileName,'r')         
    print("READING LINES...")
    lines=f.read().splitlines()        
    f.close()
    print 'Header is: ',lines[0]
    N_lines=int(len(lines))  				
    N_words=int(len(lines[1].split()))
    
    print("INPUT LINES IN NUMPY ARRAY...")
    print(N_lines,N_words)
    fileData=np.zeros((N_lines-1,N_words)) #Leaving first line which contain header
    for i in range(1,N_lines,1):
        fileData[i-1]=lines[i].split()
 
    return fileData
###############################################################################

T=3.0

B=1.0/T

def logsumexp(W):
    max_W=np.max(W)
    return max_W+np.log(np.sum(np.exp(W-max_W)))
    

def BAREqn(dA,U_F,U_R):
    global B
    
    BU_F=B*U_F
    BU_R=B*U_R
    BdA=B*dA
    
    nF=float(len(U_F))
    nR=float(len(U_R))
    
    nRatio=nF/nR
    
    LOGnRatio=np.log(nRatio)
    
    eArgF=LOGnRatio+BU_F-BdA
    
    max_eArgF=np.max(eArgF)
    
    log_eArgF = -max_eArgF - np.log(np.exp(-max_eArgF)+np.exp(eArgF-max_eArgF))
    
    logNumr=logsumexp(log_eArgF)


    eArgR=-LOGnRatio-BU_R+BdA    
    max_eArgR=np.max(eArgR)    
    log_eArgR = -max_eArgR - np.log(np.exp(-max_eArgR)+np.exp(eArgR-max_eArgR))

    logDenr=logsumexp(log_eArgR) 
    
    return logNumr-logDenr

lmbCtr=np.array(range(1,48,1))

def generateSample(s):
    numOfData=len(s)
    sample0=[]
    for i in range(numOfData):
        sample0.append(random.sample(s,1))

    sample0=np.array(sample0)
    sample0=sample0.reshape((numOfData,))
    return sample0

def BootStrap(sF,sR,numOfSample):
    X=[]
    for i in range(numOfSample):
        sampleF=generateSample(sF)
        sampleR=generateSample(sR)
        X.append(getFE(sampleF,sampleR))
    meanX=np.mean(X)
    semX=np.std(X)
    
    return meanX,semX
        
    
    
def getFE(U_F,U_R):
    return (fsolve(BAREqn,0,args=(U_F,U_R))[0])

dArr=np.zeros(len(lmbCtr)-1)
dArrOrg=np.zeros(len(lmbCtr)-1)
dErr=np.zeros(len(lmbCtr)-1)
for ctr in range(0,len(lmbCtr)-1,1):
    print "Processing state...",(ctr+1)
    FileName1=r'lmbda_pt_%s'%str(lmbCtr[ctr])+r'\EqmRun.txt'
    FileName2=r'lmbda_pt_%s'%str(lmbCtr[ctr+1])+r'\EqmRun.txt'
    CutVal=50000
    StepVal=15
    U_F=(file2array(FileName1)[:,-2])[CutVal::StepVal]
    U_R=(file2array(FileName2)[:,-1])[CutVal::StepVal]
    
    numOfSample=200
    dArrVal,dErrVal=BootStrap(U_F,U_R,numOfSample)
    dArrOrg[ctr-1]=fsolve(BAREqn,0,args=(U_F,U_R))[0]
    dArr[ctr-1]=dArrVal
    dErr[ctr-1]=dErrVal
    

dA=np.sum(dArr)
dAOrg=np.sum(dArrOrg)
dE=np.sqrt(np.sum(dErr*dErr))


print dA,dE
print "dAOrg: ",dAOrg
###############################################################################
plt.figure()
xAxis=np.array(range(0,len(lmbCtr)-1,1))
plt.errorbar(xAxis,dArr,dErr,ls='--',marker='o')
plt.ticklabel_format(useOffset=False, style='plain')
plt.xlabel(r'$\lambda_{ij}$', fontsize =36)
plt.ylabel(r'$\Delta A$', fontsize=36)
plt.rc('xtick', labelsize=30)
plt.rc('ytick', labelsize=20)
plt.tick_params(labelsize=28)

plt.axhline(0, color='black')
plt.axvline(0, color='black')
